
class ATSchemaFieldDescriptor(property):
    """A descriptor for accessing AT schema fields.

      >>> class Mock(object):
      ...     def __init__(self, **kwargs):
      ...         for key, value in kwargs.items(): setattr(self, key, value)
      >>> class MockField(object):
      ...     def __init__(self, v=''): self.attr = v
      ...     def getAccessor(self, x):
      ...         return lambda obj=self: obj.attr
      ...     def getMutator(self, x):
      ...         def set(v, obj=self):
      ...             obj.attr = v
      ...         return set
      >>> mockfield = MockField()
      >>> someattr = Mock(schema={'foobar': mockfield})
      >>> class MockTest(object):
      ...     someattr = someattr
      ...     somefield = ATSchemaFieldDescriptor('foobar', 'someattr')
      >>> obj = MockTest()

      >>> obj.somefield
      u''

      >>> obj.somefield = 'abc'
      >>> obj.somefield
      u'abc'

      >>> obj.somefield = u'someuni'
      >>> obj.somefield
      u'someuni'

    """

    def __init__(self, field, subobj_name=None, uni=True):
        self.field = field
        self.subobj_name = subobj_name
        self.ensure_unicode = uni

    def _decode(self, v):
        v = v or u''
        if not isinstance(v, unicode) and self.ensure_unicode:
            v = unicode(v, 'utf-8')
        return v

    def obj(self, obj):
        if self.subobj_name:
            obj = getattr(obj, self.subobj_name)
        return obj

    def atfield_accessor(self, obj):
        obj = self.obj(obj)
        return obj.schema[self.field].getAccessor(obj)

    def atfield_mutator(self, obj):
        obj = self.obj(obj)
        return obj.schema[self.field].getMutator(obj)

    def __get__(self, obj, type=None):
        return self._decode(self.atfield_accessor(obj)())

    def __set__(self, obj, v):
        self.atfield_mutator(obj)(self._decode(v))

    def __delete__(self, obj):
        self.atfield_mutator(obj)(None)



from zope import interface
from zope.component.interfaces import ISiteManager
from zope.app.component.interfaces import ISite

import logging
logger = logging.getLogger('p4a.common.testing')

try:
    import five.localsitemanager
    HAS_FLSM = True
    logger.info('Using five.localsitemanager')
except ImportError, err:
    HAS_FLSM = False

class MockSite:
    """A simple ISite/ISiteManager combo for testing purposes.
    """

    interface.implements(ISite, ISiteManager)

    def __init__(self):
        self.utils = {}

    def getUtility(self, iface):
        return self.utils[iface]

    def queryUtility(self, iface):
        return self.utils.get(iface, None)

    def getSiteManager(self):
        return self

    if not HAS_FLSM:
        def registerUtility(self, iface, obj):
            self.utils[iface] = obj
    else:
        def registerUtility(self, obj, iface):
            self.utils[iface] = obj



from zope import interface
from zope import event
from p4a.common import interfaces

_marker = object()

class ActivationException(Exception):
    """Any exception related to activating or deactivating a feature.
    """

def _set_activation(obj, possibleiface, enhancediface, activation):
    """Try to set the feature activation on the given object.
    """

    if isinstance(possibleiface, basestring):
        raise NotImplementedError('Still need to locaate class via dotted '
                                  'path string')
    if isinstance(enhancediface, basestring):
        raise NotImplementedError('Still need to locaate class via '
                                  'dotted path string')

    if not possibleiface.providedBy(obj):
        raise ActivationException('The object denoted by %r does not '
                                  'provide the %r interface' 
                                  % (repr(obj), repr(possibleiface)))

    if not activation and enhancediface.implementedBy(obj.__class__):
        raise ActivationException('It is not possible to remove %r from '
                                  'the %r since the class is the '
                                  'implementer'
                                  % (repr(possibleiface), repr(obj)))

    ifaces = interface.directlyProvidedBy(obj)
    if activation and not enhancediface.providedBy(obj):
        interface.alsoProvides(obj, enhancediface)
        event.notify(interfaces.FeatureActivatedEvent(enhancediface, obj))
    elif not activation and enhancediface in ifaces:
        interface.directlyProvides(obj, ifaces - enhancediface)
        event.notify(interfaces.FeatureDeactivatedEvent(enhancediface, obj))

def activate(obj, possibleiface, enhancediface):
    _set_activation(obj, possibleiface, enhancediface, True)

def deactivate(obj, possibleiface, enhancediface):
    if getattr(obj.context, 'layout', None) is not None:
        delattr(obj.context, 'layout')
    _set_activation(obj, possibleiface, enhancediface, False)

_marker = object()

class FeatureProperty(object):
    """Allows you to get and set the *feature* on the given object.
    """

    def __init__(self, possibleiface, enhancediface, attrname=None):
        self.__possibleiface = possibleiface
        self.__enhancediface = enhancediface
        self.__attrname = attrname

    def __get__(self, inst, class_):
        if inst is None:
            return self

        obj = inst
        if self.__attrname:
            obj = getattr(inst, self.__attrname)
    
        return self.__enhancediface.providedBy(obj)

    def __set__(self, inst, value):
        obj = inst
        if self.__attrname:
            obj = getattr(inst, self.__attrname)
        
        if value:
            activate(obj, self.__possibleiface, self.__enhancediface)
        else:
            deactivate(obj, self.__possibleiface, self.__enhancediface)



from zope.component.interfaces import ISiteManager
from zope.app.component.hooks import setSite
from zope.app.component.interfaces import ISite, IPossibleSite
from Products.Five.site.localsite import enableLocalSiteHook

def ensure_site(context):
    """Ensure the given context implements ISite.  The importance of
    this method is that it will ensure the given context is an ISite
    regardless of the Zope version (Zope 2.9 had a really hacked up
    SiteManager mechanism we have to account for).

      >>> from zope.app.component.interfaces import ISite, IPossibleSite
      >>> from OFS.Folder import Folder
      >>> if not IPossibleSite.implementedBy(Folder):
      ...    from zope import interface
      ...    from Products.Five.site.metaconfigure import (FiveSite,
      ...                                                  classSiteHook)
      ...    classSiteHook(Folder, FiveSite)
      ...    interface.classImplements(Folder, IPossibleSite)
      >>> om = Folder('foo')

      >>> ISite.providedBy(om)
      False

      >>> ensure_site(om)
      >>> ISite.providedBy(om)
      True

    """

    if not IPossibleSite.providedBy(context):
        if hasattr(context, 'getPhysicalPath'):
            p = '/'.join(context.getPhysicalPath())
        elif hasattr(context, 'getId'):
            p = context.getId()
        elif hasattr(context, 'id'):
            p = id
        else:
            p = str(context)

        raise TypeError('The object, "%s", is not an IPossibleSite' % p)

    if not ISite.providedBy(context):
        enableLocalSiteHook(context)
        setSite(context)

    if not ISite.providedBy(context):
        raise TypeError('Somehow trying to configure "%s" as an ISite '
                        'has failed' % '/'.join(context.getPhysicalPath()))



from zope import interface

class IFeatureStatusChangedEvent(interface.Interface):
    """Fired when a featuer has been activated or deactivated."""

    object = interface.Attribute('Object')
    enhancedinterface = interface.Attribute('Enhanced Interface')

class IFeatureActivatedEvent(IFeatureStatusChangedEvent): pass
class IFeatureDeactivatedEvent(IFeatureStatusChangedEvent): pass

class FeatureStatusChangedEvent(object):
    interface.implements(IFeatureStatusChangedEvent)

    def __init__(self, enhancedinterface, object):
        self.enhancedinterface = enhancedinterface
        self.object = object

class FeatureActivatedEvent(FeatureStatusChangedEvent):
    interface.implements(IFeatureActivatedEvent)

class FeatureDeactivatedEvent(FeatureStatusChangedEvent):
    interface.implements(IFeatureDeactivatedEvent)



from persistent.dict import PersistentDict
try:
    from zope.app.annotation import interfaces as annointerfaces
except ImportError, err:
    # Zope 2.10 support
    from zope.annotation import interfaces as annointerfaces

_marker = object()

class AnnotationDescriptor(property):
    """A descriptor for accessing annotated fields.

      >>> import zope.interface
      >>> import zope.schema
      >>> class IFoo(zope.interface.Interface):
      ...     field1 = zope.schema.Text(title=u'Field1')
      >>> class Foo(dict):
      ...     zope.interface.implements(IFoo, annointerfaces.IAnnotations)
      ...     field1 = AnnotationDescriptor('key', IFoo['field1'])

      >>> foo = Foo()
      >>> foo.field1

      >>> foo.field1 = u'abc'
      >>> foo.field1
      u'abc'

      >>> del foo.field1
      >>> foo.field1
      Traceback (most recent call last):
        ...
      AttributeError: 'field1'

    """

    def __init__(self, anno_key, field, subobj_name=None):
        self.anno_key = anno_key
        self.field = field
        self.subobj_name = subobj_name

    def __get__(self, obj, type=None):
        if self.subobj_name:
            obj = getattr(obj, self.subobj_name)
        anno = annointerfaces.IAnnotations(obj)
        name = self.field.__name__
        default = getattr(self.field, 'default', _marker)

        d = anno.get(self.anno_key, _marker)
        if d is _marker:
            if default is _marker:
                raise AttributeError(name)
            return default

        try:
            return d[name]
        except KeyError, e:
            raise AttributeError(str(e))

    def __set__(self, obj, v):
        if self.subobj_name:
            obj = getattr(obj, self.subobj_name)
        anno = annointerfaces.IAnnotations(obj)
        d = anno.get(self.anno_key, None)
        if d is None:
            d = PersistentDict()
            anno[self.anno_key] = d
        name = self.field.__name__
        d[name] = v

    def __delete__(self, obj):
        if self.subobj_name:
            obj = getattr(obj, self.subobj_name)
        anno = annointerfaces.IAnnotations(obj)
        d = anno.get(self.anno_key, None)
        name = self.field.__name__
        if d is None:
            raise AttributeError(name)
        try:
            del d[name]
        except KeyError, e:
            raise AttributeError(str(e))



import os

def package_dirs(pkg_name):
    """Return the physical filesystem directory (if possible) of the given
    package.

      >>> package_dirs('p4a.common')
      ('...p4a/common',)

    Feeding a module name will result in returning the paths of the parent
    package.

      >>> package_dirs('p4a.common.pkgutils')
      ('...p4a/common',)

    Feeding a non-existent package/module will simply raise an ImportError.

      >>> package_dirs('p4a.common.foobar')
      Traceback (most recent call last):
        ...
      ImportError: No module named foobar

    """

    pkg = __import__(pkg_name, globals(), locals(), pkg_name)
    if not hasattr(pkg, '__path__'):
        # pkg_name was actually the name of a module, lets get the actual pkg
        pkg_name = '.'.join(pkg_name.split('.')[:-1])
        pkg = __import__(pkg_name, globals(), locals(), pkg_name)

    return tuple(getattr(pkg, '__path__', ()))

def find_dir(pkg_name, dirname='.'):
    """Return an absolute path to a directory if one can be found, or None
    otherwise.  The search is recursive starting at the directories
    provided by pkg_name.

      >>> find_dir('p4a.common', '.')
      '.../p4a/common'

    The search will be recursive, in this example it will look at all pkg dirs
    provided by 'p4a' and recursively search for pkgutils.py.

      >>> find_dir('p4a', 'pkgutils.py')
      '.../p4a/common/pkgutils.py'
    """

    # special case, basically means return the first directory provided by
    # the package
    if dirname == '.':
        dirs = package_dirs(pkg_name)
        if len(dirs) > 0:
            return dirs[0]
        return None

    def find(basedir, dirname=dirname):
        for x in os.listdir(basedir):
            full = os.path.join(basedir, x)
            if x == dirname:
                return full
            elif os.path.isdir(full):
                res = find(full)
                if res:
                    return res

    for x in package_dirs(pkg_name):
        res = find(x)
        if res:
             return res

    return None



from Acquisition import aq_inner, aq_base
from zope import schema
from zope.formlib import form
from zope.app.form.browser.textwidgets import TextAreaWidget
from Products.Five.formlib.formbase import PageDisplayForm, PageForm
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore import utils as cmfutils
from Products.Archetypes import Field, Widget
from Products.ATContentTypes.content import document

class RichTextEditWidget(BrowserView, TextAreaWidget):
    """A Zope 3 based formlib widget that exposes whatever rich text
    editor is configured inside Plone.
    """

    template = ViewPageTemplateFile('atwidget.pt')

    def __init__(self, *args, **kwargs):
        BrowserView.__init__(self, *args, **kwargs)
        TextAreaWidget.__init__(self, *args, **kwargs)

    def content_context(self):
        current = aq_inner(self.context.context)
        content_context = None
        for x in range(100):
            if hasattr(current, '__of__'):
                content_context = current
                break
            if hasattr(current, 'context'):
                current = current.context
            else:
                break
        return content_context

    def __call__(self):
        self.context.REQUEST = self.request
        if not 'body' in self.request.form:
            self.request.form['body'] = self.context.get(self.context.context)
        template = aq_base(self.template)
        widget = aq_base(self)
        content_context = self.content_context()

        template = template.__of__(widget.__of__(content_context))
        return template()

    def hasInput(self):
        return 'body' in self.request.form

    def getInputValue(self):
        return self.request.form.get('body', None)

form_fields = form.FormFields(
    schema.Text(__name__='simpletext',
                title=u'Simple Text',
                required=False),
    schema.Text(__name__='richtext',
                title=u'Rich Text',
                required=False),
    )

class TestEditFieldsView(PageForm):
    """
    """

    label = u'Test Fields'
    form_fields = form_fields
    form_fields['richtext'].custom_widget = RichTextEditWidget

    @form.action('Save')
    def handle_save_action(self, action, data):
        pass

class TestDisplayFieldsView(PageDisplayForm):
    """
    """

    label = u'Test Fields'
    form_fields = form_fields

    actions = ()



from Acquisition import aq_base, aq_parent
from zope import interface
from OFS.ObjectManager import ObjectManager

def update_portal_for_z210(portal):
    """The way that the component registry and other things changed when going
    from Zope 2.9 to 2.10.  This function will migrate all aspects that it
    can migrate.
    """

    portal = aq_base(portal)
    if 'utilities' in portal.objectIds():
        for id, u in portal.utilities.objectItems():
            pieces = id.split('-')
            ifacename = pieces[0]
            name = u''
            if len(pieces) > 1:
                name = pieces[1]
            iface = None
            for checkiface in interface.providedBy(u).flattened():
                if checkiface.getName() == ifacename:
                    iface = checkiface
                    break
            if iface is not None:
                update_utility(portal, id, u, iface, name)

        if len(portal.utilities.objectIds()) == 0:
            # using ObjectManager since it doesn't do as intensive
            # security checks as PloneSite.manage_delObjects
            ObjectManager.manage_delObjects(portal, ['utilities'])

def update_utility(newsite, id, utility, iface, name=u''):
    """Take the given utility having come from a pre-zope2.10 based
    site, register it with the new site, and delete it from the old
    site.
    """

    sm = newsite.getSiteManager()
    sm.registerUtility(utility, iface, name=name)
    parent = aq_parent(utility)
    parent.manage_delObjects([id])



import unittest
from zope.testing import doctest

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite('p4a.common.formatting'),
        doctest.DocTestSuite('p4a.common.pkgutils',
                             optionflags=doctest.ELLIPSIS),
        doctest.DocTestSuite('p4a.common.site',
                             optionflags=doctest.ELLIPSIS),
        doctest.DocTestSuite('p4a.common.descriptors._anno',
                             optionflags=doctest.ELLIPSIS),
        doctest.DocTestSuite('p4a.common.descriptors._at',
                             optionflags=doctest.ELLIPSIS),
        doctest.DocTestSuite('p4a.common.dtutils',
                             optionflags=doctest.ELLIPSIS),
        ))

if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')



from setuptools import setup, find_packages

version = '1.0.5'

setup(name='p4a.common',
      version=version,
      description="Reusable code-bits for Zope 3 and Plone",
      long_description="""""",
      classifiers=[
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone4artists',
      author='Rocky Burt',
      author_email='rocky@serverzen.com',
      url='http://pypi.python.org/pypi/p4a.common/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['p4a'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          "setuptools",
          "python-dateutil",
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from datetime import datetime
from datetime import timedelta

ONE_DAY = timedelta(days=1)

def day_suffix(day):
    """
    Return correct English suffix (i.e. 'st', 'nd' etc.)

      >>> day_suffix(1)
      u'st'
      >>> day_suffix(6)
      u'th'
      >>> day_suffix(21)
      u'st'
      >>> day_suffix(26)
      u'th'

    """

    if 4 <= day <= 20 or 24 <= day <= 30:
        return u'th'
    else:
        return [u'st', u'nd', u'rd'][day % 10 - 1]

def same_day(*dates):
    """Return True if and only if each date in *dates represents the exact
    same date.

      >>> from datetime import date
      >>> same_day(date(2006, 1, 29))
      True

      >>> same_day(date(2006, 1, 29), date(2006, 1, 29))
      True

      >>> same_day(date(2006, 1, 29), date(2006, 1, 29), date(2006, 3, 22))
      False

    """
    onedate = dates[0]
    for x in dates[1:]:
        if x.year != onedate.year or x.month != onedate.month \
               or x.day != onedate.day:
            return False

    return True

def same_month(*dates):
    """Return True if and only if each date in *dates represents the exact
    same month and year.

      >>> from datetime import date
      >>> same_month(date(2006, 1, 29))
      True

      >>> same_month(date(2006, 1, 29), date(2006, 1, 29))
      True

      >>> same_month(date(2006, 1, 29), date(2006, 1, 29), date(2006, 1, 22))
      True

      >>> same_month(date(2006, 1, 29), date(2006, 5, 29))
      False

    """
    onedate = dates[0]
    for x in dates[1:]:
        if x.year != onedate.year or x.month != onedate.month:
            return False

    return True

def fancy_date_interval(start, end=None):
    """
    If dates share month and year, format it so the month
    is only displayed once.

    >>> from datetime import datetime, timedelta
    >>> fancy_date_interval(datetime(2006, 1, 29), datetime(2006, 1, 30))
    u'Jan 29th-30th, 2006'

    >>> fancy_date_interval(datetime(2006, 1, 29), datetime(2006, 2, 1))
    u'Jan 29th-Feb 1st, 2006'

    >>> fancy_date_interval(datetime(2006, 1, 29), datetime(2006, 1, 29))
    u'Jan 29th, 2006'

    >>> fancy_date_interval(datetime.today())
    u'Today'

    >>> fancy_date_interval(datetime.today()-timedelta(days=1))
    u'Yesterday'

    """

    if end is None or same_day(start, end):
        if same_day(start, datetime.today()):
            return u'Today'
        if same_day(start, datetime.today() - ONE_DAY):
            return u'Yesterday'
        return u'%s %s%s, %s' % (unicode(start.strftime("%b"),"utf-8"),
                                 start.day,
                                 day_suffix(start.day),
                                 start.year)
    elif same_month(start, end):
        return u'%s %s%s-%s%s, %s' % (unicode(start.strftime("%b"),"utf-8"),
                                      start.day,
                                      day_suffix(start.day),
                                      end.day,
                                      day_suffix(end.day),
                                      start.year)
    else:
        return u'%s %s%s-%s %s%s, %s' % (unicode(start.strftime("%b"),"utf-8"),
                                         start.day,
                                         day_suffix(start.day),
                                         end.strftime("%b"),
                                         end.day,
                                         day_suffix(end.day),
                                         start.year)

def fancy_time_amount(v, show_legend=True):
    """Produce a friendly representation of the given time amount.  The
    value is expected to be in seconds as an int.

      >>> fancy_time_amount(391)
      u'06:31 (mm:ss)'

      >>> fancy_time_amount(360)
      u'06:00 (mm:ss)'

      >>> fancy_time_amount(6360)
      u'01:46:00 (hh:mm:ss)'

      >>> fancy_time_amount(360, False)
      u'06:00'

    """

    remainder = v
    hours = remainder / 60 / 60
    remainder = remainder - (hours * 60 * 60)
    mins = remainder / 60
    secs = remainder - (mins * 60)

    if hours > 0:
        val = u'%02i:%02i:%02i' % (hours, mins, secs)
        legend =  u' (hh:mm:ss)'
    else:
        val = u'%02i:%02i' % (mins, secs)
        legend = u' (mm:ss)'

    if show_legend:
        full = val + legend
    else:
        full = val

    return full

def fancy_data_size(v):
    """Produce a friendly reprsentation of the given value.  The value
    Is expected to be in bytes as an int or long.
    
      >>> fancy_data_size(54)
      u'54 B'
      
      >>> fancy_data_size(37932)
      u'37.0 KB'

      >>> fancy_data_size(1237932)
      u'1.2 MB'
    
      >>> fancy_data_size(2911237932)
      u'2.7 GB'
    """
    
    suffix = 'B'
    
    format = u'%i %s'
    if v > 1024:
        suffix = 'KB'
        v = float(v) / 1024.0
        format = u'%.1f %s'
    
    if v > 1024:
        suffix = 'MB'
        v = v / 1024.0
    
    if v > 1024:
        suffix = 'GB'
        v = v / 1024.0

    return format % (v, suffix)

def _test():
    import doctest
    doctest.testmod()
    
if __name__ == "__main__":
    _test()
